@@ -44,10 +44,10 @@ class FileLock(Enum):
     Prefix = "pfx.lock"  # WINEPREFIX lock
 
 
-# Gamescope looks for the app ID 769 to determine if the window is Steam's
-# The app ID is expected to be in the atom GAMESCOPECTRL_BASELAYER_APPID and is
-# a required value in the sequence
-STEAM_WINDOW_ID = 769
+# Minimum size expected of the system's tmpfs when writing to it.
+# We want a tmpfs that's sufficient to fit at least Proton and the Steam
+# Linux Runtime image, which is currently ~500MB and ~200MB.
+TMPFS_MIN = 1073741824
 
 PROTON_VERBS = {
     "waitforexitandrun",
