@@ -10,7 +10,7 @@ from itertools import chain
 from pathlib import Path
 from re import split as resplit
 from shutil import move
-from tempfile import TemporaryDirectory
+from tempfile import TemporaryDirectory, mkdtemp
 from typing import Any
 
 from urllib3.exceptions import HTTPError
@@ -31,6 +31,7 @@ from umu.umu_log import log
 from umu.umu_util import (
     extract_tarfile,
     file_digest,
+    get_tempdir,
     run_zenity,
     unix_flock,
     write_file_chunks,
@@ -81,8 +82,8 @@ def get_umu_proton(env: dict[str, str], session_pools: SessionPools) -> dict[str
     except HTTPError:
         log.debug("Network is unreachable")
 
-    with TemporaryDirectory() as tmp, TemporaryDirectory(dir=UMU_CACHE) as tmpcache:
-        tmpdirs: SessionCaches = (Path(tmp), Path(tmpcache))
+    with TemporaryDirectory(dir=UMU_CACHE) as tmpcache:
+        tmpdirs: SessionCaches = (get_tempdir(), Path(tmpcache))
         compatdirs = (UMU_COMPAT, STEAM_COMPAT)
         if _get_delta(env, UMU_COMPAT, patch, assets, session_pools) is env:
             return env
@@ -259,12 +260,13 @@ def _fetch_proton(
         parts: Path = tmpfs.joinpath(f"{tarball}.parts")
         cached_parts: Path = UMU_CACHE.joinpath(parts.name)
         headers: dict[str, str] | None = None
+        has_cache: bool = cached_parts.is_file()
 
         # Resume from our cached file, if we were interrupted previously
-        if cached_parts.is_file():
+        if has_cache:
             log.info("Found '%s' in cache, resuming...", cached_parts.name)
             headers = {"Range": f"bytes={cached_parts.stat().st_size}-"}
-            parts = cached_parts
+            parts = cached_parts.rename(f"{mkdtemp(dir=UMU_CACHE)}/{parts.name}")
             # Rebuild our hashed progress
             with parts.open("rb") as fp:
                 hashsum = file_digest(fp, hashsum.name)
@@ -309,6 +311,9 @@ def _fetch_proton(
             )
             raise ValueError(err)
 
+        if has_cache:
+            move(parts, UMU_CACHE)
+
         log.info("%s: SHA512 is OK", tarball)
 
     return env
@@ -515,9 +520,11 @@ def _get_delta(
         log.debug("Received empty byte string for patch, skipping")
         return None
 
-    from cbor2 import CBORDecodeError, dumps, loads
+    # Ignore. cbor2 is an optional dep
+    from cbor2 import CBORDecodeError, dumps, loads  # noqa: PLC0415
 
-    from .umu_delta import valid_key, valid_signature
+    # Ignore. umu_delta is relevant for sys packages when using *-Latest tokens
+    from .umu_delta import valid_key, valid_signature  # noqa: PLC0415
 
     try:
         cbor = loads(patch)
