@@ -2,11 +2,10 @@ import os
 import re
 import sys
 import threading
-import time
 from _ctypes import CFuncPtr
 from argparse import Namespace
 from array import array
-from collections.abc import MutableMapping
+from collections.abc import Generator, MutableMapping
 from concurrent.futures import Future, ThreadPoolExecutor
 from contextlib import suppress
 from ctypes import CDLL, c_int, c_ulong
@@ -15,6 +14,7 @@ from itertools import chain
 from pathlib import Path
 from pwd import getpwuid
 from re import match
+from secrets import token_hex
 from socket import AF_INET, SOCK_DGRAM, socket
 from subprocess import Popen
 from typing import Any
@@ -26,7 +26,6 @@ from urllib3.util import Timeout
 from Xlib import X, Xatom, display
 from Xlib.error import DisplayConnectionError
 from Xlib.protocol.request import GetProperty
-from Xlib.protocol.rq import Event
 from Xlib.xobject.drawable import Window
 
 from umu import __runtime_versions__, __version__
@@ -34,7 +33,6 @@ from umu.umu_consts import (
     PR_SET_CHILD_SUBREAPER,
     PROTON_VERBS,
     STEAM_COMPAT,
-    STEAM_WINDOW_ID,
     UMU_LOCAL,
     FileLock,
     GamescopeAtom,
@@ -129,11 +127,8 @@ def check_env(
     if os.environ.get("UMU_NO_PROTON") == "1":
         return env
 
-    path: Path = STEAM_COMPAT.joinpath(os.environ.get("PROTONPATH", ""))
-    if os.environ.get("PROTONPATH") and path.name == "UMU-Latest":
-        path.unlink(missing_ok=True)
-
     # Proton Version
+    path: Path = STEAM_COMPAT.joinpath(os.environ.get("PROTONPATH", ""))
     if os.environ.get("PROTONPATH") and path.is_dir():
         os.environ["PROTONPATH"] = str(STEAM_COMPAT.joinpath(os.environ["PROTONPATH"]))
 
@@ -193,11 +188,10 @@ def set_env(
         # Make an absolute path to winetricks within GE-Proton or UMU-Proton.
         # The launcher will change to the winetricks parent directory before
         # creating the subprocess
-        exe: Path = Path(protonpath, "protonfixes", "winetricks").resolve(strict=True)
-        env["EXE"] = str(exe)
+        exe: Path = Path(protonpath, "protonfixes", "winetricks")
+        # Handle older protons before winetricks was added to the PATH
+        env["EXE"] = str(exe.resolve(strict=True)) if exe.is_file() else "winetricks"
         args = (env["EXE"], args[1])  # type: ignore
-        if not env["STEAM_COMPAT_INSTALL_PATH"]:
-            env["STEAM_COMPAT_INSTALL_PATH"] = str(exe.parent)
     elif is_cmd:
         try:
             # Ensure executable path is absolute, otherwise Proton will fail
@@ -221,11 +215,14 @@ def set_env(
     # UMU_ID
     env["UMU_ID"] = env["GAMEID"]
     env["STEAM_COMPAT_APP_ID"] = "0"
+    env["UMU_STEAM_GAME_ID"] = os.environ.get("SteamGameId", "")  # noqa: SIM112
 
+    # Steam Application ID
     if match(r"^umu-[\d\w]+$", env["UMU_ID"]):
         env["STEAM_COMPAT_APP_ID"] = env["UMU_ID"][env["UMU_ID"].find("-") + 1 :]
     env["SteamAppId"] = env["STEAM_COMPAT_APP_ID"]
     env["SteamGameId"] = env["SteamAppId"]
+    env["UMU_INVOCATION_ID"] = token_hex(16)
 
     # PATHS
     env["WINEPREFIX"] = str(pfx)
@@ -353,11 +350,42 @@ def build_command(
     )
 
 
-def get_window_ids(d: display.Display) -> set[str] | None:
+def _get_pids() -> Generator[int, Any, None]:
+    yield from (int(pid.name) for pid in Path("/proc").glob("*") if pid.name.isdigit())
+
+
+def get_pstree_from_pid(root_pid: int) -> set[int]:
+    """Get descendent PIDs of a PID."""
+    descendants: set[int] = set()
+    pid_to_ppid: dict[int, int] = {}
+
+    for pid in _get_pids():
+        try:
+            path = Path(f"/proc/{pid}/status")
+            with path.open(mode="r", encoding="utf-8") as file:
+                st_ppid = next(line for line in file if line.startswith("PPid:"))
+                st_ppid = st_ppid.removeprefix("PPid:").strip()
+                pid_to_ppid[pid] = int(st_ppid)
+        except (FileNotFoundError, ProcessLookupError, ValueError):
+            continue
+
+    current_pid: list[int] = [root_pid]
+    while current_pid:
+        current = current_pid.pop()
+        # Ignore. mypy flags [arg-type] due to the reuse of pid variable
+        for pid, ppid in pid_to_ppid.items():  # type: ignore
+            if ppid == current and pid not in descendants:
+                descendants.add(pid)  # type: ignore
+                current_pid.append(pid)  # type: ignore
+
+    log.debug("PID %s descendents: %s", root_pid, descendants)
+    return descendants
+
+
+def get_window_ids(d: display.Display) -> set[int] | None:
     """Get the list of window ids under the root window for a display."""
     try:
-        event: Event = d.next_event()
-        if event.type == X.CreateNotify:
+        if d.next_event().type == X.CreateNotify:
             return {child.id for child in d.screen().root.query_tree().children}
     except Exception as e:
         log.exception(e)
@@ -365,16 +393,55 @@ def get_window_ids(d: display.Display) -> set[str] | None:
     return None
 
 
+def get_pstree_window_ids(
+    d: display.Display, pstree: set[int], window_ids: set[int] | None
+) -> set[int] | None:
+    """Get a subset of window ids associated with a set of pids."""
+    game_window_ids: set[int] = set()
+    atom = d.get_atom("_NET_WM_PID", only_if_exists=True)
+
+    if not window_ids:
+        return None
+
+    # Return if _NET_WM_PID does not exist. Its atom existing will be a hard
+    # requirement as, unfortunately, we cannot fallback to X-Res because the
+    # PIDs returned would not map to the ones in the Flatpak's namespace
+    if atom == X.NONE:
+        log.warning("_NET_WM_PID does not exist, skipping window IDs")
+        return None
+
+    # Ensure only the game's windows are returned via _NET_WM_PID
+    # https://specifications.freedesktop.org/wm-spec/1.3/ar01s05.html#id-1.6.14
+    for window_id in window_ids:
+        try:
+            window: Window = d.create_resource_object("window", window_id)
+            prop = window.get_full_property(atom, Xatom.CARDINAL)
+            if not prop or not prop.value:
+                continue
+            pid = prop.value.tolist()[0]
+            if pid not in pstree:
+                log.debug(
+                    "PID %s is unrelated to app or not a descendent, skipping window ID: %s",
+                    pid,
+                    window_id,
+                )
+                continue
+            log.debug("Window ID %s has PID: %s", window_id, pid)
+            game_window_ids.add(window_id)
+        except Exception as e:
+            log.exception(e)
+
+    return game_window_ids
+
+
 def set_steam_game_property(
-    d: display.Display,
-    window_ids: set[str],
-    steam_assigned_appid: int,
+    d: display.Display, window_ids: set[int], steam_assigned_appid: int
 ) -> display.Display:
     """Set Steam's assigned app ID on a list of windows."""
     log.debug("Steam app ID: %s", steam_assigned_appid)
     for window_id in window_ids:
         try:
-            window: Window = d.create_resource_object("window", int(window_id))
+            window: Window = d.create_resource_object("window", window_id)
             window.change_property(
                 d.get_atom(GamescopeAtom.SteamGame.value),
                 Xatom.CARDINAL,
@@ -419,63 +486,6 @@ def get_gamescope_baselayer_appid(
     return None
 
 
-def rearrange_gamescope_baselayer_appid(
-    sequence: list[int],
-) -> tuple[list[int], int] | None:
-    """Rearrange the GAMESCOPECTRL_BASELAYER_APPID value retrieved from a window."""
-    rearranged: list[int] = list(sequence)
-    steam_appid: int = get_steam_appid(os.environ)
-
-    log.debug("%s: %s", GamescopeAtom.BaselayerAppId.value, sequence)
-
-    if not steam_appid:
-        # Case when the app ID can't be found from environment variables
-        # See https://github.com/Open-Wine-Components/umu-launcher/issues/318
-        log.error(
-            "Failed to acquire app ID, skipping %s rearrangement",
-            GamescopeAtom.BaselayerAppId.value,
-        )
-        return None
-
-    try:
-        rearranged.remove(steam_appid)
-    except ValueError as e:
-        # Case when the app ID isn't in GAMESCOPECTRL_BASELAYER_APPID
-        # One case this can occur is if the client overrides Steam's env vars
-        # that we get the app ID from
-        log.exception(e)
-        return None
-
-    # Steam's window should be last, while assigned app id 2nd to last
-    rearranged = [*rearranged[:-1], steam_appid, STEAM_WINDOW_ID]
-    log.debug("Rearranging %s", GamescopeAtom.BaselayerAppId.value)
-    log.debug("'%s' -> '%s'", sequence, rearranged)
-
-    return rearranged, steam_appid
-
-
-def set_gamescope_baselayer_appid(
-    d: display.Display, rearranged: list[int]
-) -> display.Display | None:
-    """Set a new gamescope GAMESCOPECTRL_BASELAYER_APPID on the primary root window."""
-    try:
-        # Intern the atom for GAMESCOPECTRL_BASELAYER_APPID
-        atom = d.get_atom(GamescopeAtom.BaselayerAppId.value)
-        # Set the property value
-        d.screen().root.change_property(atom, Xatom.CARDINAL, 32, rearranged)
-        log.debug(
-            "Successfully set %s property: %s",
-            GamescopeAtom.BaselayerAppId.value,
-            ", ".join(map(str, rearranged)),
-        )
-        return d
-    except Exception as e:
-        log.error("Error setting %s property", GamescopeAtom.BaselayerAppId.value)
-        log.exception(e)
-
-    return None
-
-
 def get_steam_appid(env: MutableMapping) -> int:
     """Get the Steam app ID from the host environment variables."""
     steam_appid: int = 0
@@ -497,73 +507,41 @@ def get_steam_appid(env: MutableMapping) -> int:
         with suppress(ValueError, IndexError):
             return int(Path(path).parts[-2])
 
+    with suppress(ValueError):
+        return int(env.get("UMU_STEAM_GAME_ID", "")) >> 32
+
     return steam_appid
 
 
-def monitor_baselayer_appid(
-    d_primary: display.Display,
-    gamescope_baselayer_sequence: list[int],
-) -> None:
-    """Monitor for broken GAMESCOPECTRL_BASELAYER_APPID values."""
-    root_primary: Window = d_primary.screen().root
-    rearranged_gamescope_baselayer: tuple[list[int], int] | None = None
-    atom = d_primary.get_atom(GamescopeAtom.BaselayerAppId.value)
-    root_primary.change_attributes(event_mask=X.PropertyChangeMask)
-
-    log.debug(
-        "Monitoring %s property for DISPLAY=%s...",
-        GamescopeAtom.BaselayerAppId.value,
-        d_primary.get_display_name(),
-    )
-
-    # Rearranged GAMESCOPECTRL_BASELAYER_APPID
-    rearranged_gamescope_baselayer = rearrange_gamescope_baselayer_appid(
-        gamescope_baselayer_sequence
-    )
-
-    # Set the rearranged GAMESCOPECTRL_BASELAYER_APPID
-    if rearranged_gamescope_baselayer:
-        rearranged, _ = rearranged_gamescope_baselayer
-        set_gamescope_baselayer_appid(d_primary, rearranged)
-        rearranged_gamescope_baselayer = None
+def _get_pstree_root_pid(env: MutableMapping) -> int:
+    # https://gitlab.steamos.cloud/steamrt/steam-runtime-tools/-/raw/main/pressure-vessel/adverb.1.md
+    target = "pv-adverb"
 
+    # Find a target process where it's descendents create game windows
+    # and return its PID
     while True:
-        event: Event = d_primary.next_event()
-        prop: GetProperty | None = None
-
-        if event.type == X.PropertyNotify and event.atom == atom:
-            prop = root_primary.get_full_property(atom, Xatom.CARDINAL)
-
-        # Check if the layer sequence has changed to the broken one
-        if prop and prop.value[-1] != STEAM_WINDOW_ID:
-            log.debug(
-                "Broken %s property detected, will rearrange...",
-                GamescopeAtom.BaselayerAppId.value,
-            )
-            log.debug(
-                "%s has atom %s: %s",
-                GamescopeAtom.BaselayerAppId.value,
-                atom,
-                prop.value,
-            )
-            rearranged_gamescope_baselayer = rearrange_gamescope_baselayer_appid(
-                prop.value
-            )
-
-        if rearranged_gamescope_baselayer:
-            rearranged, _ = rearranged_gamescope_baselayer
-            set_gamescope_baselayer_appid(d_primary, rearranged)
-            rearranged_gamescope_baselayer = None
-            continue
-
-        time.sleep(0.1)
+        st: dict[str, str] = {}
+        for pid in _get_pids():
+            try:
+                # https://docs.kernel.org/filesystems/proc.html#id10
+                path = Path(f"/proc/{pid}/status")
+                with path.open(mode="r", encoding="utf-8") as file:
+                    for line in file:
+                        key, val = line.split(":", 1)
+                        st[key] = val.strip()
+                if st["Name"] != target:
+                    continue
+                # Ensure the target process is related to the game
+                inv_id = f"UMU_INVOCATION_ID={env['UMU_INVOCATION_ID']}".encode()
+                if inv_id in Path(f"/proc/{pid}/environ").read_bytes():
+                    return int(pid)
+            except (FileNotFoundError, ProcessLookupError, ValueError):
+                continue
 
 
-def monitor_windows(
-    d_secondary: display.Display,
-) -> None:
+def monitor_windows(d_secondary: display.Display, pid: int) -> None:
     """Monitor for new windows for a display and assign them Steam's assigned app ID."""
-    window_ids: set[str] | None = None
+    window_ids: set[int] | None = None
     steam_appid: int = get_steam_appid(os.environ)
 
     log.debug(
@@ -572,8 +550,9 @@ def monitor_windows(
     )
 
     while not window_ids:
-        window_ids = get_window_ids(d_secondary)
-
+        window_ids = get_pstree_window_ids(
+            d_secondary, get_pstree_from_pid(pid), get_window_ids(d_secondary)
+        )
     set_steam_game_property(d_secondary, window_ids, steam_appid)
 
     log.debug(
@@ -583,14 +562,14 @@ def monitor_windows(
 
     # Check if the window sequence has changed
     while True:
-        current_window_ids: set[str] | None = get_window_ids(d_secondary)
-
+        current_window_ids: set[int] | None = get_pstree_window_ids(
+            d_secondary, get_pstree_from_pid(pid), get_window_ids(d_secondary)
+        )
         if not current_window_ids:
             continue
-
         if diff := current_window_ids.difference(window_ids):
-            log.debug("New window IDs detected: %s", window_ids)
-            log.debug("Current tracked windows IDs: %s", current_window_ids)
+            log.debug("New window IDs detected: %s", current_window_ids)
+            log.debug("Current tracked windows IDs: %s", window_ids)
             log.debug("Window IDs set difference: %s", diff)
             window_ids |= diff
             set_steam_game_property(d_secondary, diff, steam_appid)
@@ -627,18 +606,11 @@ def run_in_steammode(proc: Popen) -> int:
 
                 # Monitor for new windows for the DISPLAY associated with game
                 window_thread = threading.Thread(
-                    target=monitor_windows, args=(d_secondary,)
+                    target=monitor_windows,
+                    args=(d_secondary, _get_pstree_root_pid(os.environ)),
                 )
                 window_thread.daemon = True
                 window_thread.start()
-
-                # Monitor for broken GAMESCOPECTRL_BASELAYER_APPID
-                baselayer_thread = threading.Thread(
-                    target=monitor_baselayer_appid,
-                    args=(d_primary, gamescope_baselayer_sequence),
-                )
-                baselayer_thread.daemon = True
-                baselayer_thread.start()
             return proc.wait()
     except DisplayConnectionError as e:
         # Case where steamos changed its display outputs as we're currently
@@ -660,10 +632,13 @@ def run_command(command: tuple[Path | str, ...]) -> int:
         os.environ.get("XDG_CURRENT_DESKTOP") == "gamescope"
         or os.environ.get("XDG_SESSION_DESKTOP") == "gamescope"
     )
+    is_flatpak: bool = os.environ.get("container") == "flatpak"  # noqa: SIM112
     # Note: STEAM_MULTIPLE_XWAYLANDS is steam mode specific and is
     # documented to be a legacy env var.
     is_steammode: bool = (
-        is_gamescope_session and os.environ.get("STEAM_MULTIPLE_XWAYLANDS") == "1"
+        is_gamescope_session
+        and os.environ.get("STEAM_MULTIPLE_XWAYLANDS") == "1"
+        and is_flatpak
     )
 
     if not command:
@@ -849,7 +824,8 @@ def umu_run(args: Namespace | tuple[str, list[str]]) -> int:
 
     # Opt to use the system's native CA bundle rather than certifi's
     with suppress(ModuleNotFoundError):
-        import truststore
+        # Ignore. truststore is an optional dep
+        import truststore  # noqa: PLC0415
 
         truststore.inject_into_ssl()
 
