@@ -4,7 +4,7 @@ from collections.abc import Generator
 from contextlib import contextmanager
 from ctypes.util import find_library
 from fcntl import LOCK_EX, LOCK_UN, flock
-from functools import lru_cache
+from functools import cache
 from hashlib import new as hashnew
 from io import BufferedIOBase, BufferedRandom
 from pathlib import Path
@@ -13,12 +13,13 @@ from re import compile as re_compile
 from shutil import which
 from subprocess import PIPE, STDOUT, Popen, TimeoutExpired
 from tarfile import open as taropen
+from tempfile import gettempdir, mkdtemp
 from typing import Any
 
 from urllib3.response import BaseHTTPResponse
 from Xlib import display
 
-from umu.umu_consts import UMU_LOCAL, WINETRICKS_SETTINGS_VERBS
+from umu.umu_consts import TMPFS_MIN, UMU_CACHE, UMU_LOCAL, WINETRICKS_SETTINGS_VERBS
 from umu.umu_log import log
 
 
@@ -53,13 +54,13 @@ def memfdfile(name: str) -> Generator[BufferedRandom, Any, None]:
             fp.close()
 
 
-@lru_cache
+@cache
 def get_libc() -> str:
     """Find libc.so from the user's system."""
     return find_library("c") or ""
 
 
-@lru_cache
+@cache
 def get_library_paths() -> set[str]:
     """Find the shared library paths from the user's system."""
     library_paths: set[str] = set()
@@ -260,6 +261,59 @@ def write_file_chunks(
     return hasher
 
 
+def _get_lines_split(
+    path: Path, sep: str | None, maxsplit: int = -1
+) -> Generator[list[str], Any, None]:
+    with path.open(mode="r", encoding="utf-8") as file:
+        lines = iter(line.split(sep=sep, maxsplit=maxsplit) for line in file)
+        yield from (columns for columns in lines if columns)
+
+
+@cache
+def _get_supported_fs() -> set[str]:
+    # https://docs.redhat.com/en/documentation/red_hat_enterprise_linux/4/html/reference_guide/s2-proc-filesystems
+    return {line[-1] for line in _get_lines_split(Path("/proc/filesystems"), None, 2)}
+
+
+@cache
+def _fsck_path(path: Path, filesystem: str) -> bool:
+    """Validate the file system of a path."""
+    if filesystem not in _get_supported_fs():
+        log.error("Path is not a supported Linux file system: %s", filesystem)
+        return False
+
+    # https://docs.kernel.org/filesystems/proc.html#kernel-data
+    # https://docs.redhat.com/en/documentation/red_hat_enterprise_linux/4/html/introduction_to_system_administration/s4-storage-mounting-proc
+    lines = _get_lines_split(Path("/proc/mounts"), None, 5)
+    dst = str(path)
+
+    return any(line for line in lines if line[1] == dst and line[2] == filesystem)
+
+
+def get_tempdir(cache: Path = UMU_CACHE) -> Path:
+    """Get a path to a temporary directory on a tmpfs.
+
+    The temporary directory is created securely, and the path returned may be
+    on a tmpfs. For a temporary directory to be on a tmpfs, the destination
+    mount point must be >= TMPFS_MIN. Otherwise, file system at $XDG_CACHE_HOME
+    will be used to create the temporary directory.
+    """
+    tmpdir = Path(gettempdir())
+
+    # Fallback to the cache instead of the current working directory
+    # https://github.com/python/cpython/blob/f297a2292cd3c3596f21ca5914310f1f8d5d8750/Lib/tempfile.py#L175
+    if tmpdir == Path.cwd():
+        tmpdir = cache
+
+    stat = os.statvfs(tmpdir)
+    has_tmpfs_min = (
+        _fsck_path(tmpdir, "tmpfs") and stat.f_frsize * stat.f_blocks >= TMPFS_MIN
+    )
+
+    # Return without handling the case where the cache is a tmpfs
+    return Path(mkdtemp()) if has_tmpfs_min else Path(mkdtemp(prefix=".", dir=cache))
+
+
 def extract_tarfile(path: Path, dest: Path) -> Path | None:
     """Read and securely extract a compressed TAR archive to path.
 
@@ -275,7 +329,8 @@ def extract_tarfile(path: Path, dest: Path) -> Path | None:
     # See https://github.com/python/cpython/blob/b83be9c9718aac42d0d8fc689a829d6594192afa/Lib/tarfile.py#L1871
     with taropen(path, f"r:{path.suffix.removeprefix('.')}") as tar:  # type: ignore
         try:
-            from tarfile import tar_filter
+            # We require Python 3.10+ and extraction filters require 3.12+
+            from tarfile import tar_filter  # noqa: PLC0415
 
             tar.extraction_filter = tar_filter
             log.debug("Using data filter for archive")
